==> CPAN-Audit-0.14.tar.gz: CPAN-Audit-0.14/lib/CPAN/Audit/Installed.pm <==
package CPAN::Audit::Installed;
use strict;
use warnings;
use File::Find ();
use Cwd        ();

sub new {
    my $class = shift;
    my (%params) = @_;

    my $self = {};
    bless $self, $class;

    $self->{db} = $params{db};
    $self->{cb} = $params{cb};

    return $self;
}

sub find {
    my $self = shift;
    my (@inc) = @_;

    @inc = @INC unless @inc;
    @inc = grep { defined && -d $_ } map { Cwd::realpath($_) } @inc;

    my %seen;
    my @deps;

    File::Find::find(
        {
            wanted => sub {
                my $path = $File::Find::name;

                if ( $path && -f $path && m/\.pm$/ ) {
                    return unless my $module = module_from_file($path);

                    return unless my $distname = $self->{db}->{module2dist}->{$module};

                    my $dist = $self->{db}->{dists}->{$distname};
                    if ( $dist->{main_module} eq $module ) {
                        return if $seen{$module}++;

                        return unless my $version = module_version($path);

                        push @deps, { dist => $distname, version => $version };

                        if ( $self->{cb} ) {
                            $self->{cb}->(
                                {
                                    path     => $path,
                                    distname => $distname,
                                    version  => $version
                                }
                            );
                        }
                    }
                }
            },
            follow      => 1,
            follow_skip => 2,
        },
        @inc
    );

    return @deps;
}

# https://metacpan.org/source/ABELTJE/V-0.13/V.pm
sub module_version {
    my ($parsefile) = @_;

    open my $mod, '<', $parsefile or die $!;

    my $inpod = 0;
    my $result;
    local $_;
    while (<$mod>) {
        $inpod = /^=(?!cut)/ ? 1 : /^=cut/ ? 0 : $inpod;
        next if $inpod || /^\s*#/;

        chomp;
        next unless m/([\$*])(([\w\:\']*)\bVERSION)\b.*\=/;
        my $eval = qq{
            package CPAN::Audit::_version;
            no strict;

            local $1$2;
            \$$2=undef; do {
                $_
            }; \$$2
        };
        local $^W = 0;
        $result = eval($eval);
        warn "Could not eval '$eval' in $parsefile: $@" if $@;
        $result = "undef" unless defined $result;
        last;
    }
    close $mod;
    return $result;
}

sub module_from_file {
    my ($path) = @_;

    my $module;

    open my $fh, '<', $path or return;
    while ( my $line = <$fh> ) {
        if ( $line =~ m/package\s+(.*?)\s*;/ms ) {
            $module = $1;
            last;
        }
    }
    close $fh;

    return unless $module;
}

1;

==> CPAN-Audit-20220629.003.tar.gz: CPAN-Audit-20220629.003/lib/CPAN/Audit/Installed.pm <==
package CPAN::Audit::Installed;
use strict;
use warnings;
use File::Find ();
use Cwd        ();

our $VERSION = "1.001";

sub new {
    my( $class, %params ) = @_;
    bless \%params, $class;
}

sub find {
    my $self = shift;
    my (@inc) = @_;

    @inc = @INC unless @inc;
    @inc = grep { defined && -d $_ } map { Cwd::realpath($_) } @inc;

    my %seen;
    my @deps;
    push @deps, { dist => 'perl', version => $] } if $self->{include_perl};

    File::Find::find(
        {
            wanted => sub {
                my $path = $File::Find::name;
                if ( $path && -f $path && m/\.pm$/ ) {
                    return unless my $module = module_from_file($path);

                    return unless my $distname = $self->{db}->{module2dist}->{$module};

                    my $dist = $self->{db}->{dists}->{$distname};
                    if ( $dist->{main_module} eq $module ) {
                        return if $seen{$module}++;

                        return unless my $version = module_version($path);

                        push @deps, { dist => $distname, version => $version };

                        if ( $self->{cb} ) {
                            $self->{cb}->(
                                {
                                    path     => $path,
                                    distname => $distname,
                                    version  => $version
                                }
                            );
                        }
                    }
                }
            },
            follow      => 1,
            follow_skip => 2,
        },
        @inc
    );

    return @deps;
}

sub module_version {
	require Module::Extract::VERSION;
    my( $file ) = @_;

	my $version = Module::Extract::VERSION->parse_version_safely( $file );

	if( eval { $version->can('numify') } ) {
		$version = $version->numify;
	}

    return "$version";
}

sub module_from_file {
    my ($path) = @_;
    my $module;

    open my $fh, '<', $path or return;
    while ( my $line = <$fh> ) {
        if ( $line =~ m/package\s+(.*?)\s*;/ms ) {
            $module = $1;
            last;
        }
    }
    close $fh;

    return unless $module;
}

1;
